% the function is to identify a fundamental matrix F minimizing the number of outliers 
% the inliers are: (x_i,x_i') such that |x_i'*F*x_i| <= epsilon
% the outliers are: (x_i,x_i') such that |x_i'*F*x_i| > epsilon
% input X: sized by n\times 9, each column is a the kronecker product of
% x_i and x_i';
%input: lambda0: the regularization parameter for low rank
%input: MaxIter: maximum number of iterations
%input: epsilon: the bound for the distance from the inlier to the subspace
%output: FF_cons: estimated fundamental matrix
%output: ss_cons: =0, inlier, =1, outlier

% f is the vectorized F
% minimize sum s_i
% subject to
% s_i^2 = s_i
% (1-s_i)*f*X(:,i) <= (1-s_i)*epsilon
% -(1-s_i)*f*X(:,i) <= (1-s_i)*epsilon
% f'*f = 1
% F*q = 0 : q is the nonzero eigenvector of F
% q'*q = 1
%%

%X = As;

function [FF_cons,ss_cons] = Fm_seeking_qcqp(X,lambda0,MaxIter,epsilon)
n = size(X,2); % the pairs of correspondences

num_f = 9;
num_q = 3;
Dimension = 1+num_f+num_q;

mm = tril(ones(Dimension));
indmm = find(mm(:)>0);
num_mc = length(indmm);
mm(indmm) = [1:num_mc]';
indSet = mm+mm'-diag(diag(mm));
ind_f = indSet(2:num_f+1,1);
ind_q = indSet(1+num_f+1:1+num_f+num_q,1);
ind_f2 = diag(indSet(ind_f,ind_f));
ind_q2 = diag(indSet(ind_q,ind_q));

pos_f = 2:num_f+1;
pos_q = num_f+1+1:num_f+num_q+1;
pos_F = reshape(pos_f,3,3);
ind_Fq = [];
for i = 1:3
    ind_Fq(i,:) = reshape(diag(indSet(pos_q,pos_F(i,:))),1,[]);
end
%%
W = eye(size(indSet));
%MaxIter = 40;

%}

lambda(1) = lambda0;
for iter = 1:MaxIter
    cvx_begin sdp
    cvx_solver sedumi
    cvx_quiet true
    variable mc(num_mc,1)
    variable s(1,n);
    variable sfq(num_f+num_q,n)
    expression Mc(size(indSet))
    Mc = mc(indSet);
    mc(1) == 1;
    Mc == semidefinite(size(indSet));
    sum(mc(ind_f2)) == 1;
    sum(mc(ind_q2)) == 1;
    mc(ind_f(9)) >= 0;
    mc(ind_q(1)) >= 0;
    for i = 1:3
        sum(mc(ind_Fq(i,:))) == 0; %Fq = 0
    end
    for i = 1:n
        MatN{i} = [Mc,[s(i);sfq(:,i)];[s(i),sfq(:,i)'],s(i)];
        MatN{i} == semidefinite(Dimension+1);
    end
    % (1-s_i)*f*X(:,i) <= (1-s_i)*epsilon
    % -(1-s_i)*f*X(:,i) <= (1-s_i)*epsilon
    for i = 1:n
        X(:,i)'*(mc(ind_f)-sfq(1:num_f,i)) - epsilon*(1-s(i)) <= 0;
        -X(:,i)'*(mc(ind_f)-sfq(1:num_f,i)) - epsilon*(1-s(i)) <= 0;
    end
    
    obj = sum(s)+lambda(iter)*trace(W*Mc);
    minimize obj
    cvx_end
    mv = [mc;s(:);sfq(:)];
    sv = svd(Mc);
    disp(strcat('iter=',num2str(iter),', ',num2str(sv(1)),', ',num2str(sv(2)),', ', cvx_status));
    if sv(2) < 1e-4*sv(1)
        disp('Found Rank-1 Solution');
        break;
    elseif iter >= 2 && max(abs(mv-mv_old)) < 1e-3
        disp('Got Stuck');
        break;
    else
        temp0= max(1e-7*(1+1+1),sv(2));
        lambda(iter+1) = lambda(iter);
        W = inv(Mc+temp0*eye(size(indSet)));
        W = W/norm(W,2);
        mv_old = mv;
    end
end

ff_cons = normc(mc(ind_f));
ss_cons = s>1e-3;
res = X'*ff_cons;
%figure,stem(abs(res))
FF_cons = reshape(ff_cons,3,3);